@@ -15,15 +15,16 @@ from django.db import models
 from django.http import Http404
 from django.template.engine import Engine
 from django.urls import get_mod_func, get_resolver, get_urlconf, reverse
-from django.utils import six
 from django.utils.decorators import method_decorator
 from django.utils.inspect import (
     func_accepts_kwargs, func_accepts_var_args, func_has_no_args,
     get_func_full_args,
 )
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from django.views.generic import TemplateView
 
+from .utils import get_view_name
+
 # Exclude methods starting with these strings from documentation
 MODEL_METHODS_EXCLUDE = ('_', 'add_', 'delete', 'save', 'set_')
 
@@ -38,19 +39,19 @@ class BaseAdminDocsView(TemplateView):
             # Display an error message for people without docutils
             self.template_name = 'admin_doc/missing_docutils.html'
             return self.render_to_response(admin.site.each_context(request))
-        return super(BaseAdminDocsView, self).dispatch(request, *args, **kwargs)
+        return super().dispatch(request, *args, **kwargs)
 
     def get_context_data(self, **kwargs):
         kwargs.update({'root_path': reverse('admin:index')})
         kwargs.update(admin.site.each_context(self.request))
-        return super(BaseAdminDocsView, self).get_context_data(**kwargs)
+        return super().get_context_data(**kwargs)
 
 
 class BookmarkletsView(BaseAdminDocsView):
     template_name = 'admin_doc/bookmarklets.html'
 
     def get_context_data(self, **kwargs):
-        context = super(BookmarkletsView, self).get_context_data(**kwargs)
+        context = super().get_context_data(**kwargs)
         context.update({
             'admin_url': "%s://%s%s" % (
                 self.request.scheme, self.request.get_host(), context['root_path'])
@@ -89,7 +90,7 @@ class TemplateTagIndexView(BaseAdminDocsView):
                         'library': tag_library,
                     })
         kwargs.update({'tags': tags})
-        return super(TemplateTagIndexView, self).get_context_data(**kwargs)
+        return super().get_context_data(**kwargs)
 
 
 class TemplateFilterIndexView(BaseAdminDocsView):
@@ -123,36 +124,26 @@ class TemplateFilterIndexView(BaseAdminDocsView):
                         'library': tag_library,
                     })
         kwargs.update({'filters': filters})
-        return super(TemplateFilterIndexView, self).get_context_data(**kwargs)
+        return super().get_context_data(**kwargs)
 
 
 class ViewIndexView(BaseAdminDocsView):
     template_name = 'admin_doc/view_index.html'
 
-    @staticmethod
-    def _get_full_name(func):
-        mod_name = func.__module__
-        if six.PY3:
-            return '%s.%s' % (mod_name, func.__qualname__)
-        else:
-            # PY2 does not support __qualname__
-            func_name = getattr(func, '__name__', func.__class__.__name__)
-            return '%s.%s' % (mod_name, func_name)
-
     def get_context_data(self, **kwargs):
         views = []
         urlconf = import_module(settings.ROOT_URLCONF)
         view_functions = extract_views_from_urlpatterns(urlconf.urlpatterns)
         for (func, regex, namespace, name) in view_functions:
             views.append({
-                'full_name': self._get_full_name(func),
+                'full_name': get_view_name(func),
                 'url': simplify_regex(regex),
                 'url_name': ':'.join((namespace or []) + (name and [name] or [])),
                 'namespace': ':'.join((namespace or [])),
                 'name': name,
             })
         kwargs.update({'views': views})
-        return super(ViewIndexView, self).get_context_data(**kwargs)
+        return super().get_context_data(**kwargs)
 
 
 class ViewDetailView(BaseAdminDocsView):
@@ -174,13 +165,6 @@ class ViewDetailView(BaseAdminDocsView):
                 # the module and class.
                 mod, klass = get_mod_func(mod)
                 return getattr(getattr(import_module(mod), klass), func)
-            except AttributeError:
-                # PY2 generates incorrect paths for views that are methods,
-                # e.g. 'mymodule.views.ViewContainer.my_view' will be
-                # listed as 'mymodule.views.my_view' because the class name
-                # can't be detected. This causes an AttributeError when
-                # trying to resolve the view.
-                return None
 
     def get_context_data(self, **kwargs):
         view = self.kwargs['view']
@@ -200,7 +184,7 @@ class ViewDetailView(BaseAdminDocsView):
             'body': body,
             'meta': metadata,
         })
-        return super(ViewDetailView, self).get_context_data(**kwargs)
+        return super().get_context_data(**kwargs)
 
 
 class ModelIndexView(BaseAdminDocsView):
@@ -209,7 +193,7 @@ class ModelIndexView(BaseAdminDocsView):
     def get_context_data(self, **kwargs):
         m_list = [m._meta for m in apps.get_models()]
         kwargs.update({'models': m_list})
-        return super(ModelIndexView, self).get_context_data(**kwargs)
+        return super().get_context_data(**kwargs)
 
 
 class ModelDetailView(BaseAdminDocsView):
@@ -339,7 +323,7 @@ class ModelDetailView(BaseAdminDocsView):
             'fields': fields,
             'methods': methods,
         })
-        return super(ModelDetailView, self).get_context_data(**kwargs)
+        return super().get_context_data(**kwargs)
 
 
 class TemplateDetailView(BaseAdminDocsView):
@@ -372,7 +356,7 @@ class TemplateDetailView(BaseAdminDocsView):
             'name': template,
             'templates': templates,
         })
-        return super(TemplateDetailView, self).get_context_data(**kwargs)
+        return super().get_context_data(**kwargs)
 
 
 ####################
@@ -391,10 +375,11 @@ def get_return_data_type(func_name):
 
 
 def get_readable_field_data_type(field):
-    """Returns the description for a given field type, if it exists,
-    Fields' descriptions can contain format strings, which will be interpolated
-    against the values of field.__dict__ before being output."""
-
+    """
+    Return the description for a given field type, if it exists. Fields'
+    descriptions can contain format strings, which will be interpolated with
+    the values of field.__dict__ before being output.
+    """
     return field.description % field.__dict__
 
 
@@ -413,13 +398,12 @@ def extract_views_from_urlpatterns(urlpatterns, base='', namespace=None):
                 continue
             views.extend(extract_views_from_urlpatterns(
                 patterns,
-                base + p.regex.pattern,
+                base + str(p.pattern),
                 (namespace or []) + (p.namespace and [p.namespace] or [])
             ))
         elif hasattr(p, 'callback'):
             try:
-                views.append((p.callback, base + p.regex.pattern,
-                              namespace, p.name))
+                views.append((p.callback, base + str(p.pattern), namespace, p.name))
             except ViewDoesNotExist:
                 continue
         else:
