@@ -2,17 +2,16 @@
 Field classes.
 """
 
-from __future__ import unicode_literals
-
 import copy
 import datetime
 import itertools
+import math
 import os
 import re
-import sys
 import uuid
 from decimal import Decimal, DecimalException
 from io import BytesIO
+from urllib.parse import urlsplit, urlunsplit
 
 from django.core import validators
 from django.core.exceptions import ValidationError
@@ -27,13 +26,11 @@ from django.forms.widgets import (
     SplitDateTimeWidget, SplitHiddenDateTimeWidget, TextInput, TimeInput,
     URLInput,
 )
-from django.utils import formats, six
+from django.utils import formats
 from django.utils.dateparse import parse_duration
 from django.utils.duration import duration_string
-from django.utils.encoding import force_str, force_text
 from django.utils.ipv6 import clean_ipv6_address
-from django.utils.six.moves.urllib.parse import urlsplit, urlunsplit
-from django.utils.translation import ugettext_lazy as _, ungettext_lazy
+from django.utils.translation import gettext_lazy as _, ngettext_lazy
 
 __all__ = (
     'Field', 'CharField', 'IntegerField',
@@ -46,7 +43,7 @@ __all__ = (
 )
 
 
-class Field(object):
+class Field:
     widget = TextInput  # Default widget to use when rendering this type of Field.
     hidden_widget = HiddenInput  # Default widget to use when rendering this as "hidden".
     default_validators = []  # Default set of validators
@@ -57,10 +54,7 @@ class Field(object):
     }
     empty_values = list(validators.EMPTY_VALUES)
 
-    # Tracks each time a Field instance is created. Used to retain order.
-    creation_counter = 0
-
-    def __init__(self, required=True, widget=None, label=None, initial=None,
+    def __init__(self, *, required=True, widget=None, label=None, initial=None,
                  help_text='', error_messages=None, show_hidden_initial=False,
                  validators=(), localize=False, disabled=False, label_suffix=None):
         # required -- Boolean that specifies whether the field is required.
@@ -112,10 +106,6 @@ class Field(object):
 
         self.widget = widget
 
-        # Increase the creation counter, and save our local copy.
-        self.creation_counter = Field.creation_counter
-        Field.creation_counter += 1
-
         messages = {}
         for c in reversed(self.__class__.__mro__):
             messages.update(getattr(c, 'default_error_messages', {}))
@@ -124,7 +114,7 @@ class Field(object):
 
         self.validators = list(itertools.chain(self.default_validators, validators))
 
-        super(Field, self).__init__()
+        super().__init__()
 
     def prepare_value(self, value):
         return value
@@ -152,10 +142,8 @@ class Field(object):
 
     def clean(self, value):
         """
-        Validates the given value and returns its "cleaned" value as an
-        appropriate Python object.
-
-        Raises ValidationError for any errors.
+        Validate the given value and return its "cleaned" value as an
+        appropriate Python object. Raise ValidationError for any errors.
         """
         value = self.to_python(value)
         self.validate(value)
@@ -177,16 +165,14 @@ class Field(object):
 
     def widget_attrs(self, widget):
         """
-        Given a Widget instance (*not* a Widget class), returns a dictionary of
+        Given a Widget instance (*not* a Widget class), return a dictionary of
         any HTML attributes that should be added to the Widget, based on this
         Field.
         """
         return {}
 
     def has_changed(self, initial, data):
-        """
-        Return True if data differs from initial.
-        """
+        """Return True if data differs from initial."""
         # Always return False if the field is disabled since self.bound_data
         # always uses the initial value in this case.
         if self.disabled:
@@ -220,28 +206,30 @@ class Field(object):
 
 
 class CharField(Field):
-    def __init__(self, max_length=None, min_length=None, strip=True, empty_value='', *args, **kwargs):
+    def __init__(self, *, max_length=None, min_length=None, strip=True, empty_value='', **kwargs):
         self.max_length = max_length
         self.min_length = min_length
         self.strip = strip
         self.empty_value = empty_value
-        super(CharField, self).__init__(*args, **kwargs)
+        super().__init__(**kwargs)
         if min_length is not None:
             self.validators.append(validators.MinLengthValidator(int(min_length)))
         if max_length is not None:
             self.validators.append(validators.MaxLengthValidator(int(max_length)))
+        self.validators.append(validators.ProhibitNullCharactersValidator())
 
     def to_python(self, value):
-        "Returns a Unicode object."
+        """Return a string."""
+        if value not in self.empty_values:
+            value = str(value)
+            if self.strip:
+                value = value.strip()
         if value in self.empty_values:
             return self.empty_value
-        value = force_text(value)
-        if self.strip:
-            value = value.strip()
         return value
 
     def widget_attrs(self, widget):
-        attrs = super(CharField, self).widget_attrs(widget)
+        attrs = super().widget_attrs(widget)
         if self.max_length is not None and not widget.is_hidden:
             # The HTML attribute is maxlength, not max_length.
             attrs['maxlength'] = str(self.max_length)
@@ -258,12 +246,12 @@ class IntegerField(Field):
     }
     re_decimal = re.compile(r'\.0*\s*$')
 
-    def __init__(self, max_value=None, min_value=None, *args, **kwargs):
+    def __init__(self, *, max_value=None, min_value=None, **kwargs):
         self.max_value, self.min_value = max_value, min_value
         if kwargs.get('localize') and self.widget == NumberInput:
             # Localized number input is not well supported on most browsers
-            kwargs.setdefault('widget', super(IntegerField, self).widget)
-        super(IntegerField, self).__init__(*args, **kwargs)
+            kwargs.setdefault('widget', super().widget)
+        super().__init__(**kwargs)
 
         if max_value is not None:
             self.validators.append(validators.MaxValueValidator(max_value))
@@ -272,23 +260,23 @@ class IntegerField(Field):
 
     def to_python(self, value):
         """
-        Validates that int() can be called on the input. Returns the result
-        of int(). Returns None for empty values.
+        Validate that int() can be called on the input. Return the result
+        of int() or None for empty values.
         """
-        value = super(IntegerField, self).to_python(value)
+        value = super().to_python(value)
         if value in self.empty_values:
             return None
         if self.localize:
             value = formats.sanitize_separators(value)
         # Strip trailing decimal and zeros.
         try:
-            value = int(self.re_decimal.sub('', force_text(value)))
+            value = int(self.re_decimal.sub('', str(value)))
         except (ValueError, TypeError):
             raise ValidationError(self.error_messages['invalid'], code='invalid')
         return value
 
     def widget_attrs(self, widget):
-        attrs = super(IntegerField, self).widget_attrs(widget)
+        attrs = super().widget_attrs(widget)
         if isinstance(widget, NumberInput):
             if self.min_value is not None:
                 attrs['min'] = self.min_value
@@ -304,8 +292,8 @@ class FloatField(IntegerField):
 
     def to_python(self, value):
         """
-        Validates that float() can be called on the input. Returns the result
-        of float(). Returns None for empty values.
+        Validate that float() can be called on the input. Return the result
+        of float() or None for empty values.
         """
         value = super(IntegerField, self).to_python(value)
         if value in self.empty_values:
@@ -319,16 +307,14 @@ class FloatField(IntegerField):
         return value
 
     def validate(self, value):
-        super(FloatField, self).validate(value)
-
-        # Check for NaN (which is the only thing not equal to itself) and +/- infinity
-        if value != value or value in (Decimal('Inf'), Decimal('-Inf')):
+        super().validate(value)
+        if value in self.empty_values:
+            return
+        if not math.isfinite(value):
             raise ValidationError(self.error_messages['invalid'], code='invalid')
 
-        return value
-
     def widget_attrs(self, widget):
-        attrs = super(FloatField, self).widget_attrs(widget)
+        attrs = super().widget_attrs(widget)
         if isinstance(widget, NumberInput) and 'step' not in widget.attrs:
             attrs.setdefault('step', 'any')
         return attrs
@@ -339,23 +325,23 @@ class DecimalField(IntegerField):
         'invalid': _('Enter a number.'),
     }
 
-    def __init__(self, max_value=None, min_value=None, max_digits=None, decimal_places=None, *args, **kwargs):
+    def __init__(self, *, max_value=None, min_value=None, max_digits=None, decimal_places=None, **kwargs):
         self.max_digits, self.decimal_places = max_digits, decimal_places
-        super(DecimalField, self).__init__(max_value, min_value, *args, **kwargs)
+        super().__init__(max_value=max_value, min_value=min_value, **kwargs)
         self.validators.append(validators.DecimalValidator(max_digits, decimal_places))
 
     def to_python(self, value):
         """
-        Validates that the input is a decimal number. Returns a Decimal
-        instance. Returns None for empty values. Ensures that there are no more
-        than max_digits in the number, and no more than decimal_places digits
+        Validate that the input is a decimal number. Return a Decimal
+        instance or None for empty values. Ensure that there are no more
+        than max_digits in the number and no more than decimal_places digits
         after the decimal point.
         """
         if value in self.empty_values:
             return None
         if self.localize:
             value = formats.sanitize_separators(value)
-        value = force_text(value).strip()
+        value = str(value).strip()
         try:
             value = Decimal(value)
         except DecimalException:
@@ -363,17 +349,14 @@ class DecimalField(IntegerField):
         return value
 
     def validate(self, value):
-        super(DecimalField, self).validate(value)
+        super().validate(value)
         if value in self.empty_values:
             return
-        # Check for NaN, Inf and -Inf values. We can't compare directly for NaN,
-        # since it is never equal to itself. However, NaN is the only value that
-        # isn't equal to itself, so we can use this to identify NaN
-        if value != value or value == Decimal("Inf") or value == Decimal("-Inf"):
+        if not math.isfinite(value):
             raise ValidationError(self.error_messages['invalid'], code='invalid')
 
     def widget_attrs(self, widget):
-        attrs = super(DecimalField, self).widget_attrs(widget)
+        attrs = super().widget_attrs(widget)
         if isinstance(widget, NumberInput) and 'step' not in widget.attrs:
             if self.decimal_places is not None:
                 # Use exponential notation for small values since they might
@@ -387,23 +370,19 @@ class DecimalField(IntegerField):
 
 class BaseTemporalField(Field):
 
-    def __init__(self, input_formats=None, *args, **kwargs):
-        super(BaseTemporalField, self).__init__(*args, **kwargs)
+    def __init__(self, *, input_formats=None, **kwargs):
+        super().__init__(**kwargs)
         if input_formats is not None:
             self.input_formats = input_formats
 
     def to_python(self, value):
-        # Try to coerce the value to unicode.
-        unicode_value = force_text(value, strings_only=True)
-        if isinstance(unicode_value, six.text_type):
-            value = unicode_value.strip()
-        # If unicode, try to strptime against each input format.
-        if isinstance(value, six.text_type):
-            for format in self.input_formats:
-                try:
-                    return self.strptime(value, format)
-                except (ValueError, TypeError):
-                    continue
+        value = value.strip()
+        # Try to strptime against each input format.
+        for format in self.input_formats:
+            try:
+                return self.strptime(value, format)
+            except (ValueError, TypeError):
+                continue
         raise ValidationError(self.error_messages['invalid'], code='invalid')
 
     def strptime(self, value, format):
@@ -419,7 +398,7 @@ class DateField(BaseTemporalField):
 
     def to_python(self, value):
         """
-        Validates that the input can be converted to a date. Returns a Python
+        Validate that the input can be converted to a date. Return a Python
         datetime.date object.
         """
         if value in self.empty_values:
@@ -428,10 +407,10 @@ class DateField(BaseTemporalField):
             return value.date()
         if isinstance(value, datetime.date):
             return value
-        return super(DateField, self).to_python(value)
+        return super().to_python(value)
 
     def strptime(self, value, format):
-        return datetime.datetime.strptime(force_str(value), format).date()
+        return datetime.datetime.strptime(value, format).date()
 
 
 class TimeField(BaseTemporalField):
@@ -443,17 +422,17 @@ class TimeField(BaseTemporalField):
 
     def to_python(self, value):
         """
-        Validates that the input can be converted to a time. Returns a Python
+        Validate that the input can be converted to a time. Return a Python
         datetime.time object.
         """
         if value in self.empty_values:
             return None
         if isinstance(value, datetime.time):
             return value
-        return super(TimeField, self).to_python(value)
+        return super().to_python(value)
 
     def strptime(self, value, format):
-        return datetime.datetime.strptime(force_str(value), format).time()
+        return datetime.datetime.strptime(value, format).time()
 
 
 class DateTimeField(BaseTemporalField):
@@ -470,7 +449,7 @@ class DateTimeField(BaseTemporalField):
 
     def to_python(self, value):
         """
-        Validates that the input can be converted to a datetime. Returns a
+        Validate that the input can be converted to a datetime. Return a
         Python datetime.datetime object.
         """
         if value in self.empty_values:
@@ -480,11 +459,11 @@ class DateTimeField(BaseTemporalField):
         if isinstance(value, datetime.date):
             result = datetime.datetime(value.year, value.month, value.day)
             return from_current_timezone(result)
-        result = super(DateTimeField, self).to_python(value)
+        result = super().to_python(value)
         return from_current_timezone(result)
 
     def strptime(self, value, format):
-        return datetime.datetime.strptime(force_str(value), format)
+        return datetime.datetime.strptime(value, format)
 
 
 class DurationField(Field):
@@ -502,27 +481,27 @@ class DurationField(Field):
             return None
         if isinstance(value, datetime.timedelta):
             return value
-        value = parse_duration(force_text(value))
+        value = parse_duration(str(value))
         if value is None:
             raise ValidationError(self.error_messages['invalid'], code='invalid')
         return value
 
 
 class RegexField(CharField):
-    def __init__(self, regex, max_length=None, min_length=None, *args, **kwargs):
+    def __init__(self, regex, **kwargs):
         """
         regex can be either a string or a compiled regular expression object.
         """
         kwargs.setdefault('strip', False)
-        super(RegexField, self).__init__(max_length, min_length, *args, **kwargs)
+        super().__init__(**kwargs)
         self._set_regex(regex)
 
     def _get_regex(self):
         return self._regex
 
     def _set_regex(self, regex):
-        if isinstance(regex, six.string_types):
-            regex = re.compile(regex, re.UNICODE)
+        if isinstance(regex, str):
+            regex = re.compile(regex)
         self._regex = regex
         if hasattr(self, '_regex_validator') and self._regex_validator in self.validators:
             self.validators.remove(self._regex_validator)
@@ -536,8 +515,8 @@ class EmailField(CharField):
     widget = EmailInput
     default_validators = [validators.validate_email]
 
-    def __init__(self, *args, **kwargs):
-        super(EmailField, self).__init__(*args, strip=True, **kwargs)
+    def __init__(self, **kwargs):
+        super().__init__(strip=True, **kwargs)
 
 
 class FileField(Field):
@@ -546,17 +525,17 @@ class FileField(Field):
         'invalid': _("No file was submitted. Check the encoding type on the form."),
         'missing': _("No file was submitted."),
         'empty': _("The submitted file is empty."),
-        'max_length': ungettext_lazy(
+        'max_length': ngettext_lazy(
             'Ensure this filename has at most %(max)d character (it has %(length)d).',
             'Ensure this filename has at most %(max)d characters (it has %(length)d).',
             'max'),
         'contradiction': _('Please either submit a file or check the clear checkbox, not both.')
     }
 
-    def __init__(self, *args, **kwargs):
-        self.max_length = kwargs.pop('max_length', None)
-        self.allow_empty_file = kwargs.pop('allow_empty_file', False)
-        super(FileField, self).__init__(*args, **kwargs)
+    def __init__(self, *, max_length=None, allow_empty_file=False, **kwargs):
+        self.max_length = max_length
+        self.allow_empty_file = allow_empty_file
+        super().__init__(**kwargs)
 
     def to_python(self, data):
         if data in self.empty_values:
@@ -596,7 +575,7 @@ class FileField(Field):
             data = None
         if not data and initial:
             return initial
-        return super(FileField, self).clean(data)
+        return super().clean(data)
 
     def bound_data(self, data, initial):
         if data in (None, FILE_INPUT_CONTRADICTION):
@@ -604,6 +583,8 @@ class FileField(Field):
         return data
 
     def has_changed(self, initial, data):
+        if self.disabled:
+            return False
         if data is None:
             return False
         return True
@@ -620,10 +601,10 @@ class ImageField(FileField):
 
     def to_python(self, data):
         """
-        Checks that the file-upload field data contains a valid image (GIF, JPG,
-        PNG, possibly others -- whatever the Python Imaging Library supports).
+        Check that the file-upload field data contains a valid image (GIF, JPG,
+        PNG, etc. -- whatever Pillow supports).
         """
-        f = super(ImageField, self).to_python(data)
+        f = super().to_python(data)
         if f is None:
             return None
 
@@ -651,12 +632,12 @@ class ImageField(FileField):
             # Pillow doesn't detect the MIME type of all formats. In those
             # cases, content_type will be None.
             f.content_type = Image.MIME.get(image.format)
-        except Exception:
+        except Exception as exc:
             # Pillow doesn't recognize it as an image.
-            six.reraise(ValidationError, ValidationError(
+            raise ValidationError(
                 self.error_messages['invalid_image'],
                 code='invalid_image',
-            ), sys.exc_info()[2])
+            ) from exc
         if hasattr(f, 'seek') and callable(f.seek):
             f.seek(0)
         return f
@@ -669,15 +650,15 @@ class URLField(CharField):
     }
     default_validators = [validators.URLValidator()]
 
-    def __init__(self, *args, **kwargs):
-        super(URLField, self).__init__(*args, strip=True, **kwargs)
+    def __init__(self, **kwargs):
+        super().__init__(strip=True, **kwargs)
 
     def to_python(self, value):
 
         def split_url(url):
             """
-            Returns a list of url parts via ``urlparse.urlsplit`` (or raises a
-            ``ValidationError`` exception for certain).
+            Return a list of url parts via urlparse.urlsplit(), or raise
+            ValidationError for some malformed URLs.
             """
             try:
                 return list(urlsplit(url))
@@ -686,7 +667,7 @@ class URLField(CharField):
                 # misformatted URLs.
                 raise ValidationError(self.error_messages['invalid'], code='invalid')
 
-        value = super(URLField, self).to_python(value)
+        value = super().to_python(value)
         if value:
             url_fields = split_url(value)
             if not url_fields[0]:
@@ -708,22 +689,24 @@ class BooleanField(Field):
     widget = CheckboxInput
 
     def to_python(self, value):
-        """Returns a Python boolean object."""
+        """Return a Python boolean object."""
         # Explicitly check for the string 'False', which is what a hidden field
         # will submit for False. Also check for '0', since this is what
         # RadioSelect will provide. Because bool("True") == bool('1') == True,
         # we don't need to handle that explicitly.
-        if isinstance(value, six.string_types) and value.lower() in ('false', '0'):
+        if isinstance(value, str) and value.lower() in ('false', '0'):
             value = False
         else:
             value = bool(value)
-        return super(BooleanField, self).to_python(value)
+        return super().to_python(value)
 
     def validate(self, value):
         if not value and self.required:
             raise ValidationError(self.error_messages['required'], code='required')
 
     def has_changed(self, initial, data):
+        if self.disabled:
+            return False
         # Sometimes data or initial may be a string equivalent of a boolean
         # so we should run it through to_python first to get a boolean value
         return self.to_python(initial) != self.to_python(data)
@@ -731,19 +714,19 @@ class BooleanField(Field):
 
 class NullBooleanField(BooleanField):
     """
-    A field whose valid values are None, True and False. Invalid values are
-    cleaned to None.
+    A field whose valid values are None, True, and False. Clean invalid values
+    to None.
     """
     widget = NullBooleanSelect
 
     def to_python(self, value):
         """
-        Explicitly checks for the string 'True' and 'False', which is what a
+        Explicitly check for the string 'True' and 'False', which is what a
         hidden field will submit for True and False, for 'true' and 'false',
         which are likely to be returned by JavaScript serializations of forms,
         and for '1' and '0', which is what a RadioField will submit. Unlike
-        the Booleanfield we need to explicitly check for True, because we are
-        not using the bool() function
+        the Booleanfield, this field must check for True because it doesn't
+        use the bool() function.
         """
         if value in (True, 'True', 'true', '1'):
             return True
@@ -756,13 +739,12 @@ class NullBooleanField(BooleanField):
         pass
 
 
-class CallableChoiceIterator(object):
+class CallableChoiceIterator:
     def __init__(self, choices_func):
         self.choices_func = choices_func
 
     def __iter__(self):
-        for e in self.choices_func():
-            yield e
+        yield from self.choices_func()
 
 
 class ChoiceField(Field):
@@ -771,16 +753,12 @@ class ChoiceField(Field):
         'invalid_choice': _('Select a valid choice. %(value)s is not one of the available choices.'),
     }
 
-    def __init__(self, choices=(), required=True, widget=None, label=None,
-                 initial=None, help_text='', *args, **kwargs):
-        super(ChoiceField, self).__init__(
-            required=required, widget=widget, label=label, initial=initial,
-            help_text=help_text, *args, **kwargs
-        )
+    def __init__(self, *, choices=(), **kwargs):
+        super().__init__(**kwargs)
         self.choices = choices
 
     def __deepcopy__(self, memo):
-        result = super(ChoiceField, self).__deepcopy__(memo)
+        result = super().__deepcopy__(memo)
         result._choices = copy.deepcopy(self._choices, memo)
         return result
 
@@ -801,16 +779,14 @@ class ChoiceField(Field):
     choices = property(_get_choices, _set_choices)
 
     def to_python(self, value):
-        "Returns a Unicode object."
+        """Return a string."""
         if value in self.empty_values:
             return ''
-        return force_text(value)
+        return str(value)
 
     def validate(self, value):
-        """
-        Validates that the input is in self.choices.
-        """
-        super(ChoiceField, self).validate(value)
+        """Validate that the input is in self.choices."""
+        super().validate(value)
         if value and not self.valid_value(value):
             raise ValidationError(
                 self.error_messages['invalid_choice'],
@@ -819,25 +795,25 @@ class ChoiceField(Field):
             )
 
     def valid_value(self, value):
-        "Check to see if the provided value is a valid choice"
-        text_value = force_text(value)
+        """Check to see if the provided value is a valid choice."""
+        text_value = str(value)
         for k, v in self.choices:
             if isinstance(v, (list, tuple)):
                 # This is an optgroup, so look inside the group for options
                 for k2, v2 in v:
-                    if value == k2 or text_value == force_text(k2):
+                    if value == k2 or text_value == str(k2):
                         return True
             else:
-                if value == k or text_value == force_text(k):
+                if value == k or text_value == str(k):
                     return True
         return False
 
 
 class TypedChoiceField(ChoiceField):
-    def __init__(self, *args, **kwargs):
-        self.coerce = kwargs.pop('coerce', lambda val: val)
-        self.empty_value = kwargs.pop('empty_value', '')
-        super(TypedChoiceField, self).__init__(*args, **kwargs)
+    def __init__(self, *, coerce=lambda val: val, empty_value='', **kwargs):
+        self.coerce = coerce
+        self.empty_value = empty_value
+        super().__init__(**kwargs)
 
     def _coerce(self, value):
         """
@@ -856,7 +832,7 @@ class TypedChoiceField(ChoiceField):
         return value
 
     def clean(self, value):
-        value = super(TypedChoiceField, self).clean(value)
+        value = super().clean(value)
         return self._coerce(value)
 
 
@@ -873,12 +849,10 @@ class MultipleChoiceField(ChoiceField):
             return []
         elif not isinstance(value, (list, tuple)):
             raise ValidationError(self.error_messages['invalid_list'], code='invalid_list')
-        return [force_text(val) for val in value]
+        return [str(val) for val in value]
 
     def validate(self, value):
-        """
-        Validates that the input is a list or tuple.
-        """
+        """Validate that the input is a list or tuple."""
         if self.required and not value:
             raise ValidationError(self.error_messages['required'], code='required')
         # Validate that each value in the value list is in self.choices.
@@ -891,26 +865,28 @@ class MultipleChoiceField(ChoiceField):
                 )
 
     def has_changed(self, initial, data):
+        if self.disabled:
+            return False
         if initial is None:
             initial = []
         if data is None:
             data = []
         if len(initial) != len(data):
             return True
-        initial_set = set(force_text(value) for value in initial)
-        data_set = set(force_text(value) for value in data)
+        initial_set = {str(value) for value in initial}
+        data_set = {str(value) for value in data}
         return data_set != initial_set
 
 
 class TypedMultipleChoiceField(MultipleChoiceField):
-    def __init__(self, *args, **kwargs):
-        self.coerce = kwargs.pop('coerce', lambda val: val)
+    def __init__(self, *, coerce=lambda val: val, **kwargs):
+        self.coerce = coerce
         self.empty_value = kwargs.pop('empty_value', [])
-        super(TypedMultipleChoiceField, self).__init__(*args, **kwargs)
+        super().__init__(**kwargs)
 
     def _coerce(self, value):
         """
-        Validates that the values are in self.choices and can be coerced to the
+        Validate that the values are in self.choices and can be coerced to the
         right type.
         """
         if value == self.empty_value or value in self.empty_values:
@@ -928,12 +904,12 @@ class TypedMultipleChoiceField(MultipleChoiceField):
         return new_value
 
     def clean(self, value):
-        value = super(TypedMultipleChoiceField, self).clean(value)
+        value = super().clean(value)
         return self._coerce(value)
 
     def validate(self, value):
         if value != self.empty_value:
-            super(TypedMultipleChoiceField, self).validate(value)
+            super().validate(value)
         elif self.required:
             raise ValidationError(self.error_messages['required'], code='required')
 
@@ -942,8 +918,8 @@ class ComboField(Field):
     """
     A Field whose clean() method calls multiple Field clean() methods.
     """
-    def __init__(self, fields=(), *args, **kwargs):
-        super(ComboField, self).__init__(*args, **kwargs)
+    def __init__(self, fields, **kwargs):
+        super().__init__(**kwargs)
         # Set 'required' to False on the individual fields, because the
         # required validation will be handled by ComboField, not by those
         # individual fields.
@@ -953,10 +929,10 @@ class ComboField(Field):
 
     def clean(self, value):
         """
-        Validates the given value against all of self.fields, which is a
+        Validate the given value against all of self.fields, which is a
         list of Field instances.
         """
-        super(ComboField, self).clean(value)
+        super().clean(value)
         for field in self.fields:
             value = field.clean(value)
         return value
@@ -964,7 +940,7 @@ class ComboField(Field):
 
 class MultiValueField(Field):
     """
-    A Field that aggregates the logic of multiple Fields.
+    Aggregate the logic of multiple Fields.
 
     Its clean() method takes a "decompressed" list of values, which are then
     cleaned into a single value according to self.fields. Each value in
@@ -984,9 +960,9 @@ class MultiValueField(Field):
         'incomplete': _('Enter a complete value.'),
     }
 
-    def __init__(self, fields=(), *args, **kwargs):
-        self.require_all_fields = kwargs.pop('require_all_fields', True)
-        super(MultiValueField, self).__init__(*args, **kwargs)
+    def __init__(self, fields, *, require_all_fields=True, **kwargs):
+        self.require_all_fields = require_all_fields
+        super().__init__(**kwargs)
         for f in fields:
             f.error_messages.setdefault('incomplete',
                                         self.error_messages['incomplete'])
@@ -998,7 +974,7 @@ class MultiValueField(Field):
         self.fields = fields
 
     def __deepcopy__(self, memo):
-        result = super(MultiValueField, self).__deepcopy__(memo)
+        result = super().__deepcopy__(memo)
         result.fields = tuple(x.__deepcopy__(memo) for x in self.fields)
         return result
 
@@ -1007,7 +983,7 @@ class MultiValueField(Field):
 
     def clean(self, value):
         """
-        Validates every value in the given list. A value is validated against
+        Validate every value in the given list. A value is validated against
         the corresponding Field in self.fields.
 
         For example, if this MultiValueField was instantiated with
@@ -1059,7 +1035,7 @@ class MultiValueField(Field):
 
     def compress(self, data_list):
         """
-        Returns a single value for the given list of values. The values can be
+        Return a single value for the given list of values. The values can be
         assumed to be valid.
 
         For example, if this MultiValueField was instantiated with
@@ -1069,6 +1045,8 @@ class MultiValueField(Field):
         raise NotImplementedError('Subclasses must implement this method.')
 
     def has_changed(self, initial, data):
+        if self.disabled:
+            return False
         if initial is None:
             initial = ['' for x in range(0, len(data))]
         else:
@@ -1085,15 +1063,11 @@ class MultiValueField(Field):
 
 
 class FilePathField(ChoiceField):
-    def __init__(self, path, match=None, recursive=False, allow_files=True,
-                 allow_folders=False, required=True, widget=None, label=None,
-                 initial=None, help_text='', *args, **kwargs):
+    def __init__(self, path, *, match=None, recursive=False, allow_files=True,
+                 allow_folders=False, **kwargs):
         self.path, self.match, self.recursive = path, match, recursive
         self.allow_files, self.allow_folders = allow_files, allow_folders
-        super(FilePathField, self).__init__(
-            choices=(), required=required, widget=widget, label=label,
-            initial=initial, help_text=help_text, *args, **kwargs
-        )
+        super().__init__(choices=(), **kwargs)
 
         if self.required:
             self.choices = []
@@ -1106,12 +1080,12 @@ class FilePathField(ChoiceField):
         if recursive:
             for root, dirs, files in sorted(os.walk(self.path)):
                 if self.allow_files:
-                    for f in files:
+                    for f in sorted(files):
                         if self.match is None or self.match_re.search(f):
                             f = os.path.join(root, f)
                             self.choices.append((f, f.replace(path, "", 1)))
                 if self.allow_folders:
-                    for f in dirs:
+                    for f in sorted(dirs):
                         if f == '__pycache__':
                             continue
                         if self.match is None or self.match_re.search(f):
@@ -1141,7 +1115,7 @@ class SplitDateTimeField(MultiValueField):
         'invalid_time': _('Enter a valid time.'),
     }
 
-    def __init__(self, input_date_formats=None, input_time_formats=None, *args, **kwargs):
+    def __init__(self, *, input_date_formats=None, input_time_formats=None, **kwargs):
         errors = self.default_error_messages.copy()
         if 'error_messages' in kwargs:
             errors.update(kwargs['error_messages'])
@@ -1154,7 +1128,7 @@ class SplitDateTimeField(MultiValueField):
                       error_messages={'invalid': errors['invalid_time']},
                       localize=localize),
         )
-        super(SplitDateTimeField, self).__init__(fields, *args, **kwargs)
+        super().__init__(fields, **kwargs)
 
     def compress(self, data_list):
         if data_list:
@@ -1170,10 +1144,10 @@ class SplitDateTimeField(MultiValueField):
 
 
 class GenericIPAddressField(CharField):
-    def __init__(self, protocol='both', unpack_ipv4=False, *args, **kwargs):
+    def __init__(self, *, protocol='both', unpack_ipv4=False, **kwargs):
         self.unpack_ipv4 = unpack_ipv4
         self.default_validators = validators.ip_address_validators(protocol, unpack_ipv4)[0]
-        super(GenericIPAddressField, self).__init__(*args, **kwargs)
+        super().__init__(**kwargs)
 
     def to_python(self, value):
         if value in self.empty_values:
@@ -1187,11 +1161,11 @@ class GenericIPAddressField(CharField):
 class SlugField(CharField):
     default_validators = [validators.validate_slug]
 
-    def __init__(self, *args, **kwargs):
-        self.allow_unicode = kwargs.pop('allow_unicode', False)
+    def __init__(self, *, allow_unicode=False, **kwargs):
+        self.allow_unicode = allow_unicode
         if self.allow_unicode:
             self.default_validators = [validators.validate_unicode_slug]
-        super(SlugField, self).__init__(*args, **kwargs)
+        super().__init__(**kwargs)
 
 
 class UUIDField(CharField):
@@ -1205,7 +1179,7 @@ class UUIDField(CharField):
         return value
 
     def to_python(self, value):
-        value = super(UUIDField, self).to_python(value)
+        value = super().to_python(value)
         if value in self.empty_values:
             return None
         if not isinstance(value, uuid.UUID):
