@@ -2,29 +2,21 @@
 HTML Widget classes
 """
 
-from __future__ import unicode_literals
-
 import copy
 import datetime
 import re
+import warnings
 from itertools import chain
 
 from django.conf import settings
 from django.forms.utils import to_current_timezone
 from django.templatetags.static import static
-from django.utils import datetime_safe, formats, six
+from django.utils import datetime_safe, formats
 from django.utils.dates import MONTHS
-from django.utils.deprecation import (
-    RemovedInDjango20Warning, RenameMethodsBase,
-)
-from django.utils.encoding import (
-    force_str, force_text, python_2_unicode_compatible,
-)
 from django.utils.formats import get_format
 from django.utils.html import format_html, html_safe
 from django.utils.safestring import mark_safe
-from django.utils.six.moves import range
-from django.utils.translation import ugettext_lazy
+from django.utils.translation import gettext_lazy as _
 
 from .renderers import get_default_renderer
 
@@ -41,26 +33,32 @@ __all__ = (
 MEDIA_TYPES = ('css', 'js')
 
 
+class MediaOrderConflictWarning(RuntimeWarning):
+    pass
+
+
 @html_safe
-@python_2_unicode_compatible
-class Media(object):
-    def __init__(self, media=None, **kwargs):
-        if media:
-            media_attrs = media.__dict__
+class Media:
+    def __init__(self, media=None, css=None, js=None):
+        if media is not None:
+            css = getattr(media, 'css', {})
+            js = getattr(media, 'js', [])
         else:
-            media_attrs = kwargs
+            if css is None:
+                css = {}
+            if js is None:
+                js = []
+        self._css = css
+        self._js = js
 
-        self._css = {}
-        self._js = []
-
-        for name in MEDIA_TYPES:
-            getattr(self, 'add_' + name)(media_attrs.get(name))
+    def __repr__(self):
+        return 'Media(css=%r, js=%r)' % (self._css, self._js)
 
     def __str__(self):
         return self.render()
 
     def render(self):
-        return mark_safe('\n'.join(chain(*[getattr(self, 'render_' + name)() for name in MEDIA_TYPES])))
+        return mark_safe('\n'.join(chain.from_iterable(getattr(self, 'render_' + name)() for name in MEDIA_TYPES)))
 
     def render_js(self):
         return [
@@ -73,13 +71,13 @@ class Media(object):
     def render_css(self):
         # To keep rendering order consistent, we can't just iterate over items().
         # We need to sort the keys, and iterate over the sorted list.
-        media = sorted(self._css.keys())
-        return chain(*[[
+        media = sorted(self._css)
+        return chain.from_iterable([
             format_html(
                 '<link href="{}" type="text/css" media="{}" rel="stylesheet" />',
                 self.absolute_path(path), medium
             ) for path in self._css[medium]
-        ] for medium in media])
+        ] for medium in media)
 
     def absolute_path(self, path):
         """
@@ -92,29 +90,53 @@ class Media(object):
         return static(path)
 
     def __getitem__(self, name):
-        "Returns a Media object that only contains media of the given type"
+        """Return a Media object that only contains media of the given type."""
         if name in MEDIA_TYPES:
             return Media(**{str(name): getattr(self, '_' + name)})
         raise KeyError('Unknown media type "%s"' % name)
 
-    def add_js(self, data):
-        if data:
-            for path in data:
-                if path not in self._js:
-                    self._js.append(path)
+    @staticmethod
+    def merge(list_1, list_2):
+        """
+        Merge two lists while trying to keep the relative order of the elements.
+        Warn if the lists have the same two elements in a different relative
+        order.
 
-    def add_css(self, data):
-        if data:
-            for medium, paths in data.items():
-                for path in paths:
-                    if not self._css.get(medium) or path not in self._css[medium]:
-                        self._css.setdefault(medium, []).append(path)
+        For static assets it can be important to have them included in the DOM
+        in a certain order. In JavaScript you may not be able to reference a
+        global or in CSS you might want to override a style.
+        """
+        # Start with a copy of list_1.
+        combined_list = list(list_1)
+        last_insert_index = len(list_1)
+        # Walk list_2 in reverse, inserting each element into combined_list if
+        # it doesn't already exist.
+        for path in reversed(list_2):
+            try:
+                # Does path already exist in the list?
+                index = combined_list.index(path)
+            except ValueError:
+                # Add path to combined_list since it doesn't exist.
+                combined_list.insert(last_insert_index, path)
+            else:
+                if index > last_insert_index:
+                    warnings.warn(
+                        'Detected duplicate Media files in an opposite order:\n'
+                        '%s\n%s' % (combined_list[last_insert_index], combined_list[index]),
+                        MediaOrderConflictWarning,
+                    )
+                # path already exists in the list. Update last_insert_index so
+                # that the following elements are inserted in front of this one.
+                last_insert_index = index
+        return combined_list
 
     def __add__(self, other):
         combined = Media()
-        for name in MEDIA_TYPES:
-            getattr(combined, 'add_' + name)(getattr(self, '_' + name, None))
-            getattr(combined, 'add_' + name)(getattr(other, '_' + name, None))
+        combined._js = self.merge(self._js, other._js)
+        combined._css = {
+            medium: self.merge(self._css.get(medium, []), other._css.get(medium, []))
+            for medium in self._css.keys() | other._css.keys()
+        }
         return combined
 
 
@@ -159,13 +181,7 @@ class MediaDefiningClass(type):
         return new_class
 
 
-class RenameWidgetMethods(MediaDefiningClass, RenameMethodsBase):
-    renamed_methods = (
-        ('_format_value', 'format_value', RemovedInDjango20Warning),
-    )
-
-
-class Widget(six.with_metaclass(RenameWidgetMethods)):
+class Widget(metaclass=MediaDefiningClass):
     needs_multipart_form = False  # Determines does this widget need multipart form
     is_localized = False
     is_required = False
@@ -199,7 +215,7 @@ class Widget(six.with_metaclass(RenameWidgetMethods)):
             return None
         if self.is_localized:
             return formats.localize_input(value)
-        return force_text(value)
+        return str(value)
 
     def get_context(self, name, value, attrs):
         context = {}
@@ -214,9 +230,7 @@ class Widget(six.with_metaclass(RenameWidgetMethods)):
         return context
 
     def render(self, name, value, attrs=None, renderer=None):
-        """
-        Returns this Widget rendered as HTML, as a Unicode string.
-        """
+        """Render the widget as an HTML string."""
         context = self.get_context(name, value, attrs)
         return self._render(self.template_name, context, renderer)
 
@@ -226,7 +240,7 @@ class Widget(six.with_metaclass(RenameWidgetMethods)):
         return mark_safe(renderer.render(template_name, context))
 
     def build_attrs(self, base_attrs, extra_attrs=None):
-        "Helper function for building an attribute dictionary."
+        """Build an attribute dictionary."""
         attrs = base_attrs.copy()
         if extra_attrs is not None:
             attrs.update(extra_attrs)
@@ -234,8 +248,8 @@ class Widget(six.with_metaclass(RenameWidgetMethods)):
 
     def value_from_datadict(self, data, files, name):
         """
-        Given a dictionary of data and this widget's name, returns the value
-        of this widget. Returns None if it's not provided.
+        Given a dictionary of data and this widget's name, return the value
+        of this widget or None if it's not provided.
         """
         return data.get(name)
 
@@ -244,8 +258,8 @@ class Widget(six.with_metaclass(RenameWidgetMethods)):
 
     def id_for_label(self, id_):
         """
-        Returns the HTML ID attribute of this Widget for use by a <label>,
-        given the ID of the field. Returns None if no ID is available.
+        Return the HTML ID attribute of this Widget for use by a <label>,
+        given the ID of the field. Return None if no ID is available.
 
         This hook is necessary because some widgets have multiple HTML
         elements and, thus, multiple IDs. In that case, this method should
@@ -269,10 +283,10 @@ class Input(Widget):
         if attrs is not None:
             attrs = attrs.copy()
             self.input_type = attrs.pop('type', self.input_type)
-        super(Input, self).__init__(attrs)
+        super().__init__(attrs)
 
     def get_context(self, name, value, attrs):
-        context = super(Input, self).get_context(name, value, attrs)
+        context = super().get_context(name, value, attrs)
         context['widget']['type'] = self.input_type
         return context
 
@@ -302,13 +316,13 @@ class PasswordInput(Input):
     template_name = 'django/forms/widgets/password.html'
 
     def __init__(self, attrs=None, render_value=False):
-        super(PasswordInput, self).__init__(attrs)
+        super().__init__(attrs)
         self.render_value = render_value
 
     def get_context(self, name, value, attrs):
         if not self.render_value:
             value = None
-        return super(PasswordInput, self).get_context(name, value, attrs)
+        return super().get_context(name, value, attrs)
 
 
 class HiddenInput(Input):
@@ -318,13 +332,13 @@ class HiddenInput(Input):
 
 class MultipleHiddenInput(HiddenInput):
     """
-    A widget that handles <input type="hidden"> for fields that have a list
+    Handle <input type="hidden"> for fields that have a list
     of values.
     """
     template_name = 'django/forms/widgets/multiple_hidden.html'
 
     def get_context(self, name, value, attrs):
-        context = super(MultipleHiddenInput, self).get_context(name, value, attrs)
+        context = super().get_context(name, value, attrs)
         final_attrs = context['widget']['attrs']
         id_ = context['widget']['attrs'].get('id')
 
@@ -374,9 +388,9 @@ FILE_INPUT_CONTRADICTION = object()
 
 
 class ClearableFileInput(FileInput):
-    clear_checkbox_label = ugettext_lazy('Clear')
-    initial_text = ugettext_lazy('Currently')
-    input_text = ugettext_lazy('Change')
+    clear_checkbox_label = _('Clear')
+    initial_text = _('Currently')
+    input_text = _('Change')
     template_name = 'django/forms/widgets/clearable_file_input.html'
 
     def clear_checkbox_name(self, name):
@@ -406,10 +420,10 @@ class ClearableFileInput(FileInput):
             return value
 
     def get_context(self, name, value, attrs):
-        context = super(ClearableFileInput, self).get_context(name, value, attrs)
+        context = super().get_context(name, value, attrs)
         checkbox_name = self.clear_checkbox_name(name)
         checkbox_id = self.clear_checkbox_id(checkbox_name)
-        context.update({
+        context['widget'].update({
             'checkbox_name': checkbox_name,
             'checkbox_id': checkbox_id,
             'is_initial': self.is_initial(value),
@@ -420,7 +434,7 @@ class ClearableFileInput(FileInput):
         return context
 
     def value_from_datadict(self, data, files, name):
-        upload = super(ClearableFileInput, self).value_from_datadict(data, files, name)
+        upload = super().value_from_datadict(data, files, name)
         if not self.is_required and CheckboxInput().value_from_datadict(
                 data, files, self.clear_checkbox_name(name)):
 
@@ -434,11 +448,11 @@ class ClearableFileInput(FileInput):
         return upload
 
     def use_required_attribute(self, initial):
-        return super(ClearableFileInput, self).use_required_attribute(initial) and not initial
+        return super().use_required_attribute(initial) and not initial
 
     def value_omitted_from_data(self, data, files, name):
         return (
-            super(ClearableFileInput, self).value_omitted_from_data(data, files, name) and
+            super().value_omitted_from_data(data, files, name) and
             self.clear_checkbox_name(name) not in data
         )
 
@@ -451,7 +465,7 @@ class Textarea(Widget):
         default_attrs = {'cols': '40', 'rows': '10'}
         if attrs:
             default_attrs.update(attrs)
-        super(Textarea, self).__init__(default_attrs)
+        super().__init__(default_attrs)
 
 
 class DateTimeBaseInput(TextInput):
@@ -459,7 +473,7 @@ class DateTimeBaseInput(TextInput):
     supports_microseconds = False
 
     def __init__(self, attrs=None, format=None):
-        super(DateTimeBaseInput, self).__init__(attrs)
+        super().__init__(attrs)
         self.format = format if format else None
 
     def format_value(self, value):
@@ -491,7 +505,7 @@ class CheckboxInput(Input):
     template_name = 'django/forms/widgets/checkbox.html'
 
     def __init__(self, attrs=None, check_test=None):
-        super(CheckboxInput, self).__init__(attrs)
+        super().__init__(attrs)
         # check_test is a callable that takes a value and returns True
         # if the checkbox should be checked for that value.
         self.check_test = boolean_check if check_test is None else check_test
@@ -500,14 +514,14 @@ class CheckboxInput(Input):
         """Only return the 'value' attribute if value isn't empty."""
         if value is True or value is False or value is None or value == '':
             return
-        return force_text(value)
+        return str(value)
 
     def get_context(self, name, value, attrs):
         if self.check_test(value):
             if attrs is None:
                 attrs = {}
             attrs['checked'] = True
-        return super(CheckboxInput, self).get_context(name, value, attrs)
+        return super().get_context(name, value, attrs)
 
     def value_from_datadict(self, data, files, name):
         if name not in data:
@@ -517,7 +531,7 @@ class CheckboxInput(Input):
         value = data.get(name)
         # Translate true and false strings to boolean values.
         values = {'true': True, 'false': False}
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             value = values.get(value.lower(), value)
         return bool(value)
 
@@ -537,7 +551,7 @@ class ChoiceWidget(Widget):
     option_inherits_attrs = True
 
     def __init__(self, attrs=None, choices=()):
-        super(ChoiceWidget, self).__init__(attrs)
+        super().__init__(attrs)
         # choices can be any iterable, but we may need to render this widget
         # multiple times. Thus, collapse it into a list so it can be consumed
         # more than once.
@@ -556,43 +570,39 @@ class ChoiceWidget(Widget):
         options from a BoundField for choice widgets.
         """
         value = self.format_value(value)
-        for option in self.options(name, value, attrs):
-            yield option
+        yield from self.options(name, value, attrs)
 
     def options(self, name, value, attrs=None):
         """Yield a flat list of options for this widgets."""
         for group in self.optgroups(name, value, attrs):
-            for option in group[1]:
-                yield option
+            yield from group[1]
 
     def optgroups(self, name, value, attrs=None):
         """Return a list of optgroups for this widget."""
-        default = (None, [], 0)
-        groups = [default]
+        groups = []
         has_selected = False
 
-        for option_value, option_label in chain(self.choices):
+        for index, (option_value, option_label) in enumerate(self.choices):
             if option_value is None:
                 option_value = ''
 
+            subgroup = []
             if isinstance(option_label, (list, tuple)):
-                index = groups[-1][2] + 1
+                group_name = option_value
                 subindex = 0
-                subgroup = []
-                groups.append((option_value, subgroup, index))
                 choices = option_label
             else:
-                index = len(default[1])
-                subgroup = default[1]
+                group_name = None
                 subindex = None
                 choices = [(option_value, option_label)]
+            groups.append((group_name, subgroup, index))
 
             for subvalue, sublabel in choices:
                 selected = (
-                    force_text(subvalue) in value and
-                    (has_selected is False or self.allow_multiple_selected)
+                    str(subvalue) in value and
+                    (not has_selected or self.allow_multiple_selected)
                 )
-                if selected is True and has_selected is False:
+                if selected and not has_selected:
                     has_selected = True
                 subgroup.append(self.create_option(
                     name, subvalue, sublabel, selected, index,
@@ -623,7 +633,7 @@ class ChoiceWidget(Widget):
         }
 
     def get_context(self, name, value, attrs):
-        context = super(ChoiceWidget, self).get_context(name, value, attrs)
+        context = super().get_context(name, value, attrs)
         context['widget']['optgroups'] = self.optgroups(name, context['widget']['value'], attrs)
         context['wrap_label'] = True
         return context
@@ -648,9 +658,11 @@ class ChoiceWidget(Widget):
 
     def format_value(self, value):
         """Return selected values as a list."""
+        if value is None and self.allow_multiple_selected:
+            return []
         if not isinstance(value, (tuple, list)):
             value = [value]
-        return [force_text(v) if v is not None else '' for v in value]
+        return [str(v) if v is not None else '' for v in value]
 
 
 class Select(ChoiceWidget):
@@ -662,7 +674,7 @@ class Select(ChoiceWidget):
     option_inherits_attrs = False
 
     def get_context(self, name, value, attrs):
-        context = super(Select, self).get_context(name, value, attrs)
+        context = super().get_context(name, value, attrs)
         if self.allow_multiple_selected:
             context['widget']['attrs']['multiple'] = 'multiple'
         return context
@@ -671,17 +683,14 @@ class Select(ChoiceWidget):
     def _choice_has_empty_value(choice):
         """Return True if the choice's value is empty string or None."""
         value, _ = choice
-        return (
-            (isinstance(value, six.string_types) and not bool(value)) or
-            value is None
-        )
+        return (isinstance(value, str) and not bool(value)) or value is None
 
     def use_required_attribute(self, initial):
         """
         Don't render 'required' if the first <option> has a value, as that's
         invalid HTML.
         """
-        use_required_attribute = super(Select, self).use_required_attribute(initial)
+        use_required_attribute = super().use_required_attribute(initial)
         # 'required' is always okay for <select multiple>.
         if self.allow_multiple_selected:
             return use_required_attribute
@@ -696,11 +705,11 @@ class NullBooleanSelect(Select):
     """
     def __init__(self, attrs=None):
         choices = (
-            ('1', ugettext_lazy('Unknown')),
-            ('2', ugettext_lazy('Yes')),
-            ('3', ugettext_lazy('No')),
+            ('1', _('Unknown')),
+            ('2', _('Yes')),
+            ('3', _('No')),
         )
-        super(NullBooleanSelect, self).__init__(attrs, choices)
+        super().__init__(attrs, choices)
 
     def format_value(self, value):
         try:
@@ -765,7 +774,7 @@ class CheckboxSelectMultiple(ChoiceWidget):
         """
         if index is None:
             return ''
-        return super(CheckboxSelectMultiple, self).id_for_label(id_, index)
+        return super().id_for_label(id_, index)
 
 
 class MultiWidget(Widget):
@@ -782,14 +791,14 @@ class MultiWidget(Widget):
 
     def __init__(self, widgets, attrs=None):
         self.widgets = [w() if isinstance(w, type) else w for w in widgets]
-        super(MultiWidget, self).__init__(attrs)
+        super().__init__(attrs)
 
     @property
     def is_hidden(self):
         return all(w.is_hidden for w in self.widgets)
 
     def get_context(self, name, value, attrs):
-        context = super(MultiWidget, self).get_context(name, value, attrs)
+        context = super().get_context(name, value, attrs)
         if self.is_localized:
             for widget in self.widgets:
                 widget.is_localized = self.is_localized
@@ -835,14 +844,17 @@ class MultiWidget(Widget):
 
     def decompress(self, value):
         """
-        Returns a list of decompressed values for the given compressed value.
+        Return a list of decompressed values for the given compressed value.
         The given value can be assumed to be valid, but not necessarily
         non-empty.
         """
         raise NotImplementedError('Subclasses must implement this method.')
 
     def _get_media(self):
-        "Media for a multiwidget is the combination of all media of the subwidgets"
+        """
+        Media for a multiwidget is the combination of all media of the
+        subwidgets.
+        """
         media = Media()
         for w in self.widgets:
             media = media + w.media
@@ -850,7 +862,7 @@ class MultiWidget(Widget):
     media = property(_get_media)
 
     def __deepcopy__(self, memo):
-        obj = super(MultiWidget, self).__deepcopy__(memo)
+        obj = super().__deepcopy__(memo)
         obj.widgets = copy.deepcopy(self.widgets)
         return obj
 
@@ -861,17 +873,23 @@ class MultiWidget(Widget):
 
 class SplitDateTimeWidget(MultiWidget):
     """
-    A Widget that splits datetime input into two <input type="text"> boxes.
+    A widget that splits datetime input into two <input type="text"> boxes.
     """
     supports_microseconds = False
     template_name = 'django/forms/widgets/splitdatetime.html'
 
-    def __init__(self, attrs=None, date_format=None, time_format=None):
+    def __init__(self, attrs=None, date_format=None, time_format=None, date_attrs=None, time_attrs=None):
         widgets = (
-            DateInput(attrs=attrs, format=date_format),
-            TimeInput(attrs=attrs, format=time_format),
+            DateInput(
+                attrs=attrs if date_attrs is None else date_attrs,
+                format=date_format,
+            ),
+            TimeInput(
+                attrs=attrs if time_attrs is None else time_attrs,
+                format=time_format,
+            ),
         )
-        super(SplitDateTimeWidget, self).__init__(widgets, attrs)
+        super().__init__(widgets)
 
     def decompress(self, value):
         if value:
@@ -882,19 +900,19 @@ class SplitDateTimeWidget(MultiWidget):
 
 class SplitHiddenDateTimeWidget(SplitDateTimeWidget):
     """
-    A Widget that splits datetime input into two <input type="hidden"> inputs.
+    A widget that splits datetime input into two <input type="hidden"> inputs.
     """
     template_name = 'django/forms/widgets/splithiddendatetime.html'
 
-    def __init__(self, attrs=None, date_format=None, time_format=None):
-        super(SplitHiddenDateTimeWidget, self).__init__(attrs, date_format, time_format)
+    def __init__(self, attrs=None, date_format=None, time_format=None, date_attrs=None, time_attrs=None):
+        super().__init__(attrs, date_format, time_format, date_attrs, time_attrs)
         for widget in self.widgets:
             widget.input_type = 'hidden'
 
 
 class SelectDateWidget(Widget):
     """
-    A Widget that splits date input into three <select> boxes.
+    A widget that splits date input into three <select> boxes.
 
     This also serves as an example of a Widget that has more than one HTML
     element and hence implements value_from_datadict.
@@ -906,7 +924,7 @@ class SelectDateWidget(Widget):
     template_name = 'django/forms/widgets/select_date.html'
     input_type = 'select'
     select_widget = Select
-    date_re = re.compile(r'(\d{4})-(\d\d?)-(\d\d?)$')
+    date_re = re.compile(r'(\d{4}|0)-(\d\d?)-(\d\d?)$')
 
     def __init__(self, attrs=None, years=None, months=None, empty_label=None):
         self.attrs = attrs or {}
@@ -941,10 +959,10 @@ class SelectDateWidget(Widget):
             self.day_none_value = self.none_value
 
     def get_context(self, name, value, attrs):
-        context = super(SelectDateWidget, self).get_context(name, value, attrs)
+        context = super().get_context(name, value, attrs)
         date_context = {}
-        year_choices = [(i, i) for i in self.years]
-        if self.is_required is False:
+        year_choices = [(i, str(i)) for i in self.years]
+        if not self.is_required:
             year_choices.insert(0, self.year_none_value)
         year_attrs = context['widget']['attrs'].copy()
         year_name = self.year_field % name
@@ -955,7 +973,7 @@ class SelectDateWidget(Widget):
             attrs=year_attrs,
         )
         month_choices = list(self.months.items())
-        if self.is_required is False:
+        if not self.is_required:
             month_choices.insert(0, self.month_none_value)
         month_attrs = context['widget']['attrs'].copy()
         month_name = self.month_field % name
@@ -966,7 +984,7 @@ class SelectDateWidget(Widget):
             attrs=month_attrs,
         )
         day_choices = [(i, i) for i in range(1, 32)]
-        if self.is_required is False:
+        if not self.is_required:
             day_choices.insert(0, self.day_none_value)
         day_attrs = context['widget']['attrs'].copy()
         day_name = self.day_field % name
@@ -991,14 +1009,15 @@ class SelectDateWidget(Widget):
         year, month, day = None, None, None
         if isinstance(value, (datetime.date, datetime.datetime)):
             year, month, day = value.year, value.month, value.day
-        elif isinstance(value, six.string_types):
+        elif isinstance(value, str):
             if settings.USE_L10N:
+                input_format = get_format('DATE_INPUT_FORMATS')[0]
                 try:
-                    input_format = get_format('DATE_INPUT_FORMATS')[0]
-                    d = datetime.datetime.strptime(force_str(value), input_format)
-                    year, month, day = d.year, d.month, d.day
+                    d = datetime.datetime.strptime(value, input_format)
                 except ValueError:
                     pass
+                else:
+                    year, month, day = d.year, d.month, d.day
             match = self.date_re.match(value)
             if match:
                 year, month, day = [int(val) for val in match.groups()]
@@ -1023,8 +1042,7 @@ class SelectDateWidget(Widget):
     def id_for_label(self, id_):
         for first_select in self._parse_date_fmt():
             return '%s_%s' % (id_, first_select)
-        else:
-            return '%s_month' % id_
+        return '%s_month' % id_
 
     def value_from_datadict(self, data, files, name):
         y = data.get(self.year_field % name)
