@@ -1,5 +1,6 @@
+import functools
+
 from django.core.exceptions import ImproperlyConfigured
-from django.utils import lru_cache, six
 from django.utils.functional import cached_property
 from django.utils.module_loading import import_string
 
@@ -9,7 +10,7 @@ from .exceptions import TemplateDoesNotExist
 from .library import import_library
 
 
-class Engine(object):
+class Engine:
     default_builtins = [
         'django.template.defaulttags',
         'django.template.defaultfilters',
@@ -52,12 +53,11 @@ class Engine(object):
         self.template_builtins = self.get_template_builtins(self.builtins)
 
     @staticmethod
-    @lru_cache.lru_cache()
+    @functools.lru_cache()
     def get_default():
         """
-        When only one DjangoTemplates backend is configured, returns it.
-
-        Raises ImproperlyConfigured otherwise.
+        Return the first DjangoTemplates backend that's configured, or raise
+        ImproperlyConfigured if none are configured.
 
         This is required for preserving historical APIs that rely on a
         globally available, implicitly configured engine such as:
@@ -73,18 +73,10 @@ class Engine(object):
         # local imports are required to avoid import loops.
         from django.template import engines
         from django.template.backends.django import DjangoTemplates
-        django_engines = [engine for engine in engines.all()
-                          if isinstance(engine, DjangoTemplates)]
-        if len(django_engines) == 1:
-            # Unwrap the Engine instance inside DjangoTemplates
-            return django_engines[0].engine
-        elif len(django_engines) == 0:
-            raise ImproperlyConfigured(
-                "No DjangoTemplates backend is configured.")
-        else:
-            raise ImproperlyConfigured(
-                "Several DjangoTemplates backends are configured. "
-                "You must select one explicitly.")
+        for engine in engines.all():
+            if isinstance(engine, DjangoTemplates):
+                return engine.engine
+        raise ImproperlyConfigured('No DjangoTemplates backend is configured.')
 
     @cached_property
     def template_context_processors(self):
@@ -120,7 +112,7 @@ class Engine(object):
         else:
             args = []
 
-        if isinstance(loader, six.string_types):
+        if isinstance(loader, str):
             loader_class = import_string(loader)
             return loader_class(self, *args)
         else:
@@ -130,33 +122,23 @@ class Engine(object):
     def find_template(self, name, dirs=None, skip=None):
         tried = []
         for loader in self.template_loaders:
-            if loader.supports_recursion:
-                try:
-                    template = loader.get_template(
-                        name, template_dirs=dirs, skip=skip,
-                    )
-                    return template, template.origin
-                except TemplateDoesNotExist as e:
-                    tried.extend(e.tried)
-            else:
-                # RemovedInDjango20Warning: Use old api for non-recursive
-                # loaders.
-                try:
-                    return loader(name, dirs)
-                except TemplateDoesNotExist:
-                    pass
+            try:
+                template = loader.get_template(name, skip=skip)
+                return template, template.origin
+            except TemplateDoesNotExist as e:
+                tried.extend(e.tried)
         raise TemplateDoesNotExist(name, tried=tried)
 
     def from_string(self, template_code):
         """
-        Returns a compiled Template object for the given template code,
+        Return a compiled Template object for the given template code,
         handling template inheritance recursively.
         """
         return Template(template_code, engine=self)
 
     def get_template(self, template_name):
         """
-        Returns a compiled Template object for the given template name,
+        Return a compiled Template object for the given template name,
         handling template inheritance recursively.
         """
         template, origin = self.find_template(template_name)
@@ -183,7 +165,7 @@ class Engine(object):
 
     def select_template(self, template_name_list):
         """
-        Given a list of template names, returns the first that can be loaded.
+        Given a list of template names, return the first that can be loaded.
         """
         if not template_name_list:
             raise TemplateDoesNotExist("No template names provided")
