@@ -2,8 +2,6 @@
 Views and functions for serving static files. These are only to be used
 during development, and SHOULD NOT be used in a production setting.
 """
-from __future__ import unicode_literals
-
 import mimetypes
 import os
 import posixpath
@@ -16,8 +14,7 @@ from django.http import (
 from django.template import Context, Engine, TemplateDoesNotExist, loader
 from django.utils._os import safe_join
 from django.utils.http import http_date, parse_http_date
-from django.utils.six.moves.urllib.parse import unquote
-from django.utils.translation import ugettext as _, ugettext_lazy
+from django.utils.translation import gettext as _, gettext_lazy
 
 
 def serve(request, path, document_root=None, show_indexes=False):
@@ -36,7 +33,7 @@ def serve(request, path, document_root=None, show_indexes=False):
     but if you'd like to override it, you can create a template called
     ``static/directory_index.html``.
     """
-    path = posixpath.normpath(unquote(path)).lstrip('/')
+    path = posixpath.normpath(path).lstrip('/')
     fullpath = safe_join(document_root, path)
     if os.path.isdir(fullpath):
         if show_indexes:
@@ -83,7 +80,7 @@ DEFAULT_DIRECTORY_INDEX_TEMPLATE = """
   </body>
 </html>
 """
-template_translatable = ugettext_lazy("Index of %(directory)s")
+template_translatable = gettext_lazy("Index of %(directory)s")
 
 
 def directory_index(path, fullpath):
@@ -94,13 +91,16 @@ def directory_index(path, fullpath):
         ])
     except TemplateDoesNotExist:
         t = Engine(libraries={'i18n': 'django.templatetags.i18n'}).from_string(DEFAULT_DIRECTORY_INDEX_TEMPLATE)
+        c = Context()
+    else:
+        c = {}
     files = []
     for f in os.listdir(fullpath):
         if not f.startswith('.'):
             if os.path.isdir(os.path.join(fullpath, f)):
                 f += '/'
             files.append(f)
-    c = Context({
+    c.update({
         'directory': path + '/',
         'file_list': files,
     })
